package cn.com.qmth.am.handle;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.ibatis.type.BaseTypeHandler;
import org.apache.ibatis.type.JdbcType;
import org.apache.ibatis.type.MappedJdbcTypes;
import org.apache.ibatis.type.MappedTypes;

import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.xiaoymin.knife4j.core.util.StrUtil;
import com.qmth.boot.core.exception.StatusException;

@MappedJdbcTypes(JdbcType.VARCHAR)
@MappedTypes({List.class})
public abstract class ListTypeHandler<T> extends BaseTypeHandler<List<T>> {
	protected ObjectMapper mapper = new ObjectMapper();
    @Override
    public void setNonNullParameter(PreparedStatement ps, int i, List<T> parameter, JdbcType jdbcType) throws SQLException {
		try {
			String content = CollectionUtils.isEmpty(parameter) ? null : mapper.writeValueAsString(parameter);
			ps.setString(i, content);
		} catch (Exception e) {
			throw new StatusException("出错",e);
		}
    }
 
    @Override
    public List<T> getNullableResult(ResultSet rs, String columnName) throws SQLException {
        return this.getListByJsonArrayString(rs.getString(columnName));
    }
 
    @Override
    public List<T> getNullableResult(ResultSet rs, int columnIndex) throws SQLException {
        return this.getListByJsonArrayString(rs.getString(columnIndex));
    }
 
    @Override
    public List<T> getNullableResult(CallableStatement cs, int columnIndex) throws SQLException {
        return this.getListByJsonArrayString(cs.getString(columnIndex));
    }
 
 
    private List<T> getListByJsonArrayString(String content) {
        try {
			return StrUtil.isBlank(content) ? null : mapper.readValue(content,specificType());
		} catch (Exception e) {
			throw new StatusException("出错",e);
		}
    }
 
    /**
     * 具体类型，由子类提供
     *
     * @return 具体类型
     */
    protected abstract JavaType specificType();
 
 
}
