package cn.com.qmth.am.task;

import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.annotation.PostConstruct;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import com.qmth.boot.core.concurrent.service.ConcurrentService;

import cn.com.qmth.am.bean.ModelSpeed;
import cn.com.qmth.am.config.SysProperty;
import cn.com.qmth.am.consumer.MarkingConsumer;
import cn.com.qmth.am.entity.StudentScoreEntity;
import cn.com.qmth.am.enums.LockType;
import cn.com.qmth.am.service.StudentScoreService;
import cn.com.qmth.am.utils.SpringContextHolder;

@Service
public class AiMarkingJob {

    @Autowired
    private StudentScoreService studentScoreService;

    @Autowired
    private ConcurrentService concurrentService;

    @Autowired
    private SysProperty sysProperty;

    private ExecutorService executor;

    @Autowired
    private ModelSpeed modelSpeed;

    @PostConstruct
    public void initExecutor() {
        int threadCount = sysProperty.getMarkingThreadCount();
        executor = new ThreadPoolExecutor(threadCount, threadCount, 0L, TimeUnit.SECONDS,
                new LinkedBlockingQueue<>(threadCount * 2), r -> {
                    Thread t = new Thread(r);
                    return t;
                }, (r, executor) -> {
                    if (!executor.isShutdown()) {
                        try {
                            executor.getQueue().put(r);
                        } catch (InterruptedException e) {
                            throw new RuntimeException(e);
                        }
                    }
                });
    }

    @Scheduled(fixedDelay = 1 * 1000, initialDelay = 5 * 1000)
    public void doJob() {
        if (!sysProperty.getMarkingTaskEnable()) {
            return;
        }
        boolean lock = concurrentService.getReadWriteLock(LockType.AI_MARKING.name()).writeLock().tryLock();
        try {
            if (!lock) {
                return;
            }
            this.dispose();
        } finally {
            if (lock) {
                concurrentService.getReadWriteLock(LockType.AI_MARKING.name()).writeLock().unlock();
            }
        }
    }

    private void dispose() {
        List<StudentScoreEntity> scores = studentScoreService.findAllToAiMarking();
        if (CollectionUtils.isNotEmpty(scores)) {
            modelSpeed.markingStart();
            CountDownLatch endGate = new CountDownLatch(scores.size());
            for (StudentScoreEntity score : scores) {
                if (!sysProperty.getMarkingTaskEnable()) {
                    return;
                }
                MarkingConsumer com = SpringContextHolder.getBean(MarkingConsumer.class);
                com.setScore(score);
                com.setEndGate(endGate);
                executor.execute(com);
            }
            try {
                endGate.await();
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
            modelSpeed.markingEnd();
        }
    }

}
