package cn.com.qmth.am.task;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.annotation.PostConstruct;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import com.qmth.boot.core.concurrent.service.ConcurrentService;

import cn.com.qmth.am.bean.ModelSpeed;
import cn.com.qmth.am.bean.OcrDto;
import cn.com.qmth.am.config.OcrServerCfg;
import cn.com.qmth.am.config.SysProperty;
import cn.com.qmth.am.consumer.OcrConsumer;
import cn.com.qmth.am.entity.QuestionEntity;
import cn.com.qmth.am.entity.StudentScoreEntity;
import cn.com.qmth.am.enums.LockType;
import cn.com.qmth.am.service.QuestionService;
import cn.com.qmth.am.service.StudentScoreService;
import cn.com.qmth.am.utils.SpringContextHolder;

@Service
public class OcrJob {

    @Autowired
    private StudentScoreService studentScoreService;

    @Autowired
    private QuestionService questionService;

    @Autowired
    private ConcurrentService concurrentService;

    @Autowired
    private SysProperty sysProperty;

    @Autowired
    private OcrServerCfg ocrServerCfg;

    @Autowired
    private ModelSpeed modelSpeed;

    private ExecutorService executor;

    @PostConstruct
    public void initExecutor() {
        int threadCount = sysProperty.getOcrThreadCount();
        executor = new ThreadPoolExecutor(threadCount, threadCount, 0L, TimeUnit.SECONDS,
                new LinkedBlockingQueue<>(threadCount * 2), r -> {
                    Thread t = new Thread(r);
                    return t;
                }, (r, executor) -> {
                    if (!executor.isShutdown()) {
                        try {
                            executor.getQueue().put(r);
                        } catch (InterruptedException e) {
                            throw new RuntimeException(e);
                        }
                    }
                });
    }

    @Scheduled(fixedDelay = 5 * 1000, initialDelay = 5 * 1000)
    public void doJob() {
        if (!sysProperty.getOcrTaskEnable() || CollectionUtils.isEmpty(ocrServerCfg.getOcrConfig())) {
            return;
        }

        List<QuestionEntity> qs = questionService.list();
        if (CollectionUtils.isEmpty(qs)) {
            return;
        }
        List<StudentScoreEntity> scores = studentScoreService.findAllToOcr();
        if (CollectionUtils.isEmpty(scores)) {
            return;
        }
        boolean lock = concurrentService.getReadWriteLock(LockType.OCR.name()).writeLock().tryLock();
        try {
            if (!lock) {
                return;
            }
            modelSpeed.ocrStart();
            this.dispose(scores, qs);
            modelSpeed.ocrEnd();
        } finally {
            if (lock) {
                concurrentService.getReadWriteLock(LockType.OCR.name()).writeLock().unlock();
            }
        }
    }

    private void dispose(List<StudentScoreEntity> scores, List<QuestionEntity> qs) {
        Map<Long, QuestionEntity> qmap = new HashMap<>();
        for (QuestionEntity q : qs) {
            qmap.put(q.getId(), q);
        }
        int index = 0;
        CountDownLatch endGate = new CountDownLatch(scores.size());
        for (StudentScoreEntity score : scores) {
            if (!sysProperty.getOcrTaskEnable()) {
                return;
            }
            QuestionEntity q = qmap.get(score.getQuestionId());
            if (q != null) {
                OcrConsumer com = SpringContextHolder.getBean(OcrConsumer.class);
                OcrDto dto = new OcrDto(score, q, ocrServerCfg.getOcrConfig().get(index));
                com.setOcrDto(dto);
                com.setEndGate(endGate);
                executor.execute(com);
                index = (index + 1) % ocrServerCfg.getOcrConfig().size();
            }
        }
        try {
            endGate.await();
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

}
