package cn.com.qmth.am.task;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.annotation.PostConstruct;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import com.qmth.boot.core.concurrent.service.ConcurrentService;

import cn.com.qmth.am.bean.StudentScoreVo;
import cn.com.qmth.am.consumer.UpdateMarkingScoreConsumer;
import cn.com.qmth.am.entity.QuestionEntity;
import cn.com.qmth.am.enums.LockType;
import cn.com.qmth.am.service.QuestionService;
import cn.com.qmth.am.service.StudentScoreService;
import cn.com.qmth.am.utils.SpringContextHolder;

@Service
public class StudentScoreImportJob {

    @Autowired
    private QuestionService questionService;

    @Autowired
    private StudentScoreService studentScoreService;

    @Autowired
    private ConcurrentService concurrentService;

    private ExecutorService executor;

    @PostConstruct
    public void initExecutor() {
        int threadCount = 8;
        executor = new ThreadPoolExecutor(threadCount, threadCount, 0L, TimeUnit.SECONDS,
                new LinkedBlockingQueue<>(threadCount * 2), r -> {
                    Thread t = new Thread(r);
                    return t;
                }, (r, executor) -> {
                    if (!executor.isShutdown()) {
                        try {
                            executor.getQueue().put(r);
                        } catch (InterruptedException e) {
                            throw new RuntimeException(e);
                        }
                    }
                });
    }

    @Scheduled(fixedDelay = 5 * 1000, initialDelay = 5 * 1000)
    public void doJob() {
        boolean lock = concurrentService.getReadWriteLock(LockType.MARKING_SCORE_IMPORT.name()).writeLock().tryLock();
        try {
            if (!lock) {
                return;
            }
            List<QuestionEntity> qs = questionService.list();
            if (CollectionUtils.isEmpty(qs)) {
                return;
            }
            Map<Long, QuestionEntity> qmap = new LinkedHashMap<>();
            for (QuestionEntity q : qs) {
                qmap.put(q.getId(), q);
            }
            List<StudentScoreVo> vos = studentScoreService.getAllInfoForUpdateScore();
            if (CollectionUtils.isEmpty(vos)) {
                return;
            }
            CountDownLatch endGate = new CountDownLatch(vos.size());
            for (StudentScoreVo vo : vos) {
                UpdateMarkingScoreConsumer com = SpringContextHolder.getBean(UpdateMarkingScoreConsumer.class);
                com.setVo(vo);
                com.setQuestion(qmap.get(vo.getQuestionId()));
                com.setEndGate(endGate);
                executor.execute(com);
            }
            try {
                endGate.await();
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
        } finally {
            if (lock) {
                concurrentService.getReadWriteLock(LockType.MARKING_SCORE_IMPORT.name()).writeLock().unlock();
            }
        }
    }

}
